
#include <stdafx.h>
#include <cutil.h>
#include <stdio.h>
#define THREAD 32

extern __shared__ int smSource[];

__global__ void AddTestKernel(int *destData, int *sourceData, int elements)
{
	smSource[threadIdx.x] = 0;

	int x = blockDim.x * blockIdx.x + threadIdx.x;
	if (x < elements) {
		smSource[threadIdx.x] = sourceData[x];
		__syncthreads();  //problem may occurs

		destData[x] = smSource[threadIdx.x];
	}
}
//---------------------------------------------------
void AddTest(int elements, int device, int threads)
{
	cudaSetDevice(device);
	cudaSetDevice(device);
	cudaDeviceProp deviceProp;
	cudaGetDeviceProperties(&deviceProp, device);
	printf(" Device %d: \"%s\"\n\n", device, deviceProp.name);

	int *hostSourceData = (int*)malloc(elements * sizeof(int));
	for (int i = 0; i < elements; i++) {
		hostSourceData[i] = i;
	}

	dim3 block = dim3(threads, 1, 1);
	dim3 grid = dim3((elements + block.x - 1) / block.x, 1, 1);

	int *dest = NULL;
	cudaMalloc((void**)&dest, elements * sizeof(int));
	int *source = NULL;
	cudaMalloc((void**)&source, elements * sizeof(int));
	cudaMemcpy(source, hostSourceData, sizeof(int) * elements, cudaMemcpyHostToDevice);

	AddTestKernel<<<grid, block, sizeof(int) * threads>>>(dest, source, elements);
	cudaThreadSynchronize();
	
	int *hostDestData = (int*)malloc(sizeof(int) * elements);
	cudaMemcpy(hostDestData, dest, sizeof(int) * elements, cudaMemcpyDeviceToHost);
	//result
	int result = 0;
	for (int i = 0; i < elements; i++) {
		int temp = hostSourceData[i] - hostDestData[i];
		if (temp != 0) result++;
	}
	printf("%d\n", result);

	free(hostSourceData);
	free(hostDestData);
	cudaFree(source);
	cudaFree(dest);
	printf("CUDA error:		%s\n", cudaGetErrorString(cudaGetLastError()));
}
